@@ -1,4 +1,4 @@
-from typing import Any, Dict, List, Optional, Tuple, Type, Union, cast
+from typing import Any, Dict, Generic, List, Optional, Tuple, Type, Union
 
 from blacksmith.domain.exceptions import (
     NoContractException,
@@ -14,10 +14,15 @@ from blacksmith.domain.model import (
     Response,
     ResponseBox,
 )
-from blacksmith.domain.model.params import AbstractCollectionParser
+from blacksmith.domain.model.params import (
+    AbstractCollectionParser,
+    TCollectionResponse,
+    TResponse,
+)
 from blacksmith.domain.registry import ApiRoutes, HttpCollection, HttpResource
-from blacksmith.middleware._async.base import AsyncHTTPMiddleware, AsyncMiddleware
-from blacksmith.typing import ClientName, HttpMethod, Path, ResourceName, Url
+from blacksmith.domain.typing import AsyncMiddleware
+from blacksmith.middleware._async.base import AsyncHTTPMiddleware
+from blacksmith.typing import ClientName, HTTPMethod, Path, ResourceName, Url
 
 from .base import AsyncAbstractTransport
 
@@ -34,7 +39,7 @@ def build_timeout(timeout: ClientTimeout) -> HTTPTimeout:
     return timeout
 
 
-class AsyncRouteProxy:
+class AsyncRouteProxy(Generic[TCollectionResponse, TResponse]):
     """Proxy from resource to its associate routes."""
 
     client_name: ClientName
@@ -68,7 +73,7 @@ class AsyncRouteProxy:
 
     def _prepare_request(
         self,
-        method: HttpMethod,
+        method: HTTPMethod,
         params: Union[Optional[Request], Dict[Any, Any]],
         resource: Optional[HttpResource],
     ) -> Tuple[Path, HTTPRequest, Optional[Type[Response]]]:
@@ -84,19 +89,22 @@ class AsyncRouteProxy:
             params = param_schema()
         elif not isinstance(params, param_schema):
             raise WrongRequestTypeException(
-                params.__class__, method, self.name, self.client_name
+                params.__class__,  # type: ignore
+                method,
+                self.name,
+                self.client_name,
             )
-        req = params.to_http_request(self.endpoint + resource.path)
+        req = params.to_http_request(method, self.endpoint + resource.path)
         return (resource.path, req, return_schema)
 
     def _prepare_response(
         self,
         response: HTTPResponse,
         response_schema: Optional[Type[Response]],
-        method: HttpMethod,
+        method: HTTPMethod,
         path: Path,
-    ) -> ResponseBox:
-        return ResponseBox(
+    ) -> ResponseBox[TResponse]:
+        return ResponseBox[TResponse](
             response,
             response_schema,
             method,
@@ -110,74 +118,64 @@ class AsyncRouteProxy:
         response: HTTPResponse,
         response_schema: Optional[Type[Response]],
         collection_parser: Optional[Type[AbstractCollectionParser]],
-    ) -> CollectionIterator:
+    ) -> CollectionIterator[TCollectionResponse]:
 
         return CollectionIterator(
             response, response_schema, collection_parser or self.collection_parser
         )
 
     async def _handle_req_with_middlewares(
-        self,
-        method: HttpMethod,
-        req: HTTPRequest,
-        timeout: HTTPTimeout,
-        path: Path,
+        self, req: HTTPRequest, timeout: HTTPTimeout, path: Path
     ) -> HTTPResponse:
-        async def handle_req(
-            req: HTTPRequest, method: HttpMethod, client_name: ClientName, path: Path
-        ) -> HTTPResponse:
-            return await self.transport.request(method, req, timeout)
-
-        next = cast(AsyncMiddleware, handle_req)
-
+        next: AsyncMiddleware = self.transport
         for middleware in self.middlewares:
             next = middleware(next)
 
-        resp = await next(req, method, self.client_name, path)
+        resp = await next(req, self.client_name, path, timeout)
         return resp
 
     async def _yield_collection_request(
         self,
-        method: HttpMethod,
+        method: HTTPMethod,
         params: Union[Optional[Request], Dict[Any, Any]],
         timeout: HTTPTimeout,
         collection: HttpCollection,
-    ) -> CollectionIterator:
+    ) -> CollectionIterator[TCollectionResponse]:
         path, req, resp_schema = self._prepare_request(method, params, collection)
-        resp = await self._handle_req_with_middlewares(method, req, timeout, path)
+        resp = await self._handle_req_with_middlewares(req, timeout, path)
         return self._prepare_collection_response(
             resp, resp_schema, collection.collection_parser
         )
 
     async def _collection_request(
         self,
-        method: HttpMethod,
+        method: HTTPMethod,
         params: Union[Request, Dict[Any, Any]],
         timeout: HTTPTimeout,
-    ) -> ResponseBox:
+    ) -> ResponseBox[TResponse]:
         path, req, resp_schema = self._prepare_request(
             method, params, self.routes.collection
         )
-        resp = await self._handle_req_with_middlewares(method, req, timeout, path)
+        resp = await self._handle_req_with_middlewares(req, timeout, path)
         return self._prepare_response(resp, resp_schema, method, path)
 
     async def _request(
         self,
-        method: HttpMethod,
+        method: HTTPMethod,
         params: Union[Request, Dict[Any, Any]],
         timeout: HTTPTimeout,
-    ) -> ResponseBox:
+    ) -> ResponseBox[TResponse]:
         path, req, resp_schema = self._prepare_request(
             method, params, self.routes.resource
         )
-        resp = await self._handle_req_with_middlewares(method, req, timeout, path)
+        resp = await self._handle_req_with_middlewares(req, timeout, path)
         return self._prepare_response(resp, resp_schema, method, path)
 
     async def collection_head(
         self,
         params: Union[Request, Dict[Any, Any]],
         timeout: Optional[ClientTimeout] = None,
-    ) -> ResponseBox:
+    ) -> ResponseBox[TResponse]:
         return await self._collection_request(
             "HEAD", params, build_timeout(timeout or self.timeout)
         )
@@ -186,7 +184,7 @@ class AsyncRouteProxy:
         self,
         params: Union[Optional[Request], Dict[Any, Any]] = None,
         timeout: Optional[ClientTimeout] = None,
-    ) -> CollectionIterator:
+    ) -> CollectionIterator[TCollectionResponse]:
         if not self.routes.collection:
             raise UnregisteredRouteException("GET", self.name, self.client_name)
         return await self._yield_collection_request(
@@ -200,7 +198,7 @@ class AsyncRouteProxy:
         self,
         params: Union[Request, Dict[Any, Any]],
         timeout: Optional[ClientTimeout] = None,
-    ) -> ResponseBox:
+    ) -> ResponseBox[TResponse]:
         return await self._collection_request(
             "POST", params, build_timeout(timeout or self.timeout)
         )
@@ -209,7 +207,7 @@ class AsyncRouteProxy:
         self,
         params: Union[Request, Dict[Any, Any]],
         timeout: Optional[ClientTimeout] = None,
-    ) -> ResponseBox:
+    ) -> ResponseBox[TResponse]:
         return await self._collection_request(
             "PUT", params, build_timeout(timeout or self.timeout)
         )
@@ -218,7 +216,7 @@ class AsyncRouteProxy:
         self,
         params: Union[Request, Dict[Any, Any]],
         timeout: Optional[ClientTimeout] = None,
-    ) -> ResponseBox:
+    ) -> ResponseBox[TResponse]:
         return await self._collection_request(
             "PATCH", params, build_timeout(timeout or self.timeout)
         )
@@ -227,7 +225,7 @@ class AsyncRouteProxy:
         self,
         params: Union[Request, Dict[Any, Any]],
         timeout: Optional[ClientTimeout] = None,
-    ) -> ResponseBox:
+    ) -> ResponseBox[TResponse]:
         return await self._collection_request(
             "DELETE", params, build_timeout(timeout or self.timeout)
         )
@@ -236,7 +234,7 @@ class AsyncRouteProxy:
         self,
         params: Union[Request, Dict[Any, Any]],
         timeout: Optional[ClientTimeout] = None,
-    ) -> ResponseBox:
+    ) -> ResponseBox[TResponse]:
         return await self._collection_request(
             "OPTIONS", params, build_timeout(timeout or self.timeout)
         )
@@ -245,7 +243,7 @@ class AsyncRouteProxy:
         self,
         params: Union[Request, Dict[Any, Any]],
         timeout: Optional[ClientTimeout] = None,
-    ) -> ResponseBox:
+    ) -> ResponseBox[TResponse]:
         return await self._request(
             "HEAD", params, build_timeout(timeout or self.timeout)
         )
@@ -254,7 +252,7 @@ class AsyncRouteProxy:
         self,
         params: Union[Request, Dict[Any, Any]],
         timeout: Optional[ClientTimeout] = None,
-    ) -> ResponseBox:
+    ) -> ResponseBox[TResponse]:
         resp = await self._request(
             "GET", params, build_timeout(timeout or self.timeout)
         )
@@ -264,7 +262,7 @@ class AsyncRouteProxy:
         self,
         params: Union[Request, Dict[Any, Any]],
         timeout: Optional[ClientTimeout] = None,
-    ) -> ResponseBox:
+    ) -> ResponseBox[TResponse]:
         return await self._request(
             "POST", params, build_timeout(timeout or self.timeout)
         )
@@ -273,7 +271,7 @@ class AsyncRouteProxy:
         self,
         params: Union[Request, Dict[Any, Any]],
         timeout: Optional[ClientTimeout] = None,
-    ) -> ResponseBox:
+    ) -> ResponseBox[TResponse]:
         return await self._request(
             "PUT", params, build_timeout(timeout or self.timeout)
         )
@@ -282,7 +280,7 @@ class AsyncRouteProxy:
         self,
         params: Union[Request, Dict[Any, Any]],
         timeout: Optional[ClientTimeout] = None,
-    ) -> ResponseBox:
+    ) -> ResponseBox[TResponse]:
         return await self._request(
             "PATCH", params, build_timeout(timeout or self.timeout)
         )
@@ -291,7 +289,7 @@ class AsyncRouteProxy:
         self,
         params: Union[Request, Dict[Any, Any]],
         timeout: Optional[ClientTimeout] = None,
-    ) -> ResponseBox:
+    ) -> ResponseBox[TResponse]:
         return await self._request(
             "DELETE", params, build_timeout(timeout or self.timeout)
         )
@@ -300,7 +298,7 @@ class AsyncRouteProxy:
         self,
         params: Union[Request, Dict[Any, Any]],
         timeout: Optional[ClientTimeout] = None,
-    ) -> ResponseBox:
+    ) -> ResponseBox[TResponse]:
         return await self._request(
             "OPTIONS", params, build_timeout(timeout or self.timeout)
         )
